# Generated by Django 4.0.1 on 2022-02-16 10:39

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('paper_uploads_cloudinary', '0005_auto_20211116_0840'),
    ]

    operations = [
        migrations.RenameField(
            model_name='cloudinaryfile',
            old_name='basename',
            new_name='resource_name',
        ),
        migrations.RenameField(
            model_name='cloudinaryfileitem',
            old_name='basename',
            new_name='resource_name',
        ),
        migrations.RenameField(
            model_name='cloudinaryimage',
            old_name='basename',
            new_name='resource_name',
        ),
        migrations.RenameField(
            model_name='cloudinaryimageitem',
            old_name='basename',
            new_name='resource_name',
        ),
        migrations.RenameField(
            model_name='cloudinarymedia',
            old_name='basename',
            new_name='resource_name',
        ),
        migrations.RenameField(
            model_name='cloudinarymediaitem',
            old_name='basename',
            new_name='resource_name',
        ),
    ]



# Generated by Django 3.2.6 on 2021-09-10 11:12

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('paper_uploads_cloudinary', '0002_auto_20210407_0627'),
    ]

    operations = [
        migrations.DeleteModel(
            name='CloudinaryCollection',
        ),
    ]



from typing import Type

from django.apps import apps
from django.core.management import BaseCommand
from django.db import DEFAULT_DB_ALIAS

from ... import helpers
from ...models.base import FileResource
from ...models.collection import CollectionBase, CollectionItemBase
from ...models.mixins import BacklinkModelMixin


class Command(BaseCommand):
    help = """
    Проверка целостности экземпляров файловых моделей.
    """
    options = None
    verbosity = None
    database = DEFAULT_DB_ALIAS

    def add_arguments(self, parser):
        parser.add_argument(
            "--database",
            action="store",
            dest="database",
            default=DEFAULT_DB_ALIAS,
            help="Nominates the database to use. Defaults to the 'default' database.",
        )
        parser.add_argument(
            "-o", "--check-ownership",
            action="store_true",
            default=False,
            help="Check `owner_XXX` fields.",
        )
        parser.add_argument(
            "-f", "--check-files",
            action="store_true",
            default=False,
            help="Check file existence.",
        )
        parser.add_argument(
            "-t", "--check-item-types",
            action="store_true",
            default=False,
            help="Check item `type` values.",
        )

    def _check_model_owners(self, model: Type[BacklinkModelMixin]):
        for instance in model.objects.using(self.database).iterator():
            invalid = False
            message = "The following errors were found in '{}.{}' (ID: {instance.pk}):".format(
                type(instance)._meta.app_label,
                type(instance).__name__,
                instance=instance,
            )

            owner_model = instance.get_owner_model()
            if owner_model is None:
                invalid = True
                message += "\n  Owner model '{}.{}' doesn't exists".format(
                    instance.owner_app_label,
                    instance.owner_model_name,
                )
            else:
                owner_field = instance.get_owner_field()
                if owner_field is None:
                    invalid = True
                    message += "\n  Owner model '{}.{}' has no field named '{}'".format(
                        instance.owner_app_label,
                        instance.owner_model_name,
                        instance.owner_fieldname,
                    )
                else:
                    try:
                        owner_model._base_manager.get(
                            (instance.owner_fieldname, instance.pk)
                        )
                    except owner_model.DoesNotExist:
                        invalid = True
                        message += "\n  Owner instance '{}.{}' not found".format(
                            instance.owner_app_label,
                            instance.owner_model_name,
                        )
                    except owner_model.MultipleObjectsReturned:
                        invalid = True
                        message += "\n  Multiple owners"

            if invalid:
                self.stdout.write(self.style.ERROR(message))

    def check_owners(self):
        """
        Проверяет, что ресурсы и коллекции
        *) имеют значения owner_app_label и owner_model_name, ссылающиеся на существующую модель
        *) имеют в поле owner_fieldname название поля, объявленного в модели-владельце
        *) имеют существующий экземпляр модели-владельца, ссылающийся на данный файл / коллекцию,
           и этот экземпляр единственный
        """
        if self.verbosity >= 2:
            self.stdout.write(self.style.SUCCESS("Checking resource ownership..."))

        for node in helpers.get_resource_model_trees(include_proxy=False):
            model = node.model

            if not issubclass(model, BacklinkModelMixin):
                continue

            self._check_model_owners(model)

        for model in apps.get_models():
            if not issubclass(model, CollectionBase):
                continue

            if model._meta.proxy:
                continue

            self._check_model_owners(model)

    def _check_file_existence(self, model: Type[FileResource]):
        for instance in model.objects.using(self.database).iterator():
            invalid = False
            message = "The following errors were found in '{}.{}' (ID: {instance.pk}):".format(
                type(instance)._meta.app_label,
                type(instance).__name__,
                instance=instance
            )

            if not instance.file_exists():
                invalid = True
                message += "\n  File missing: {}".format(instance.name)

            if invalid:
                self.stdout.write(self.style.ERROR(message))

    def check_file_existence(self):
        """
        Проверяет, что экземпляры загруженных файлов (UploadedFile, UploadedImage и т.п.)
        и элементов коллекций ссылаются на существующие файлы.

        P.S.: Не проверяет существование вариаций изображений!
        """
        if self.verbosity >= 2:
            self.stdout.write(self.style.SUCCESS("Checking file existence..."))

        for node in helpers.get_resource_model_trees(include_proxy=False):
            model = node.model

            if not issubclass(model, FileResource):
                continue

            self._check_file_existence(model)

    def check_item_types(self):
        """
        Проверяет, что элементы коллекций
        *) имеют значение type, которое присутствует в коллекции
        *) имеют класс, соответствующий модели, указанной для данного type
        """
        if self.verbosity >= 2:
            self.stdout.write(self.style.SUCCESS("Checking item type values..."))

        for item in CollectionItemBase.objects.using(self.database).iterator():
            invalid = False
            message = "The following errors were found in '{}.{}' (ID: {item.pk}):".format(
                type(item)._meta.app_label,
                type(item).__name__,
                item=item
            )

            collection_cls = item.get_collection_class()
            if item.type not in collection_cls.item_types:
                invalid = True
                message += "\n  Item type '{}' is not defined in collection '{}.{}' (ID: {})".format(
                    item.type,
                    collection_cls._meta.app_label,
                    collection_cls.__name__,
                    item.collection_id,
                )
            else:
                item_model = collection_cls.get_item_model(item.type)
                if item_model is not type(item):
                    invalid = True
                    message += "\n  Item class '{}.{}' differs from '{}.{}' defined for '{}' item type".format(
                        item._meta.app_label,
                        item.__name__,
                        item_model._meta.app_label,
                        item_model.__name__,
                        item.type
                    )

            if invalid:
                self.stdout.write(self.style.ERROR(message))

    def handle(self, *args, **options):
        self.options = options
        self.verbosity = options["verbosity"]
        self.database = options["database"]

        check_ownership = options["check_ownership"]
        check_files = options["check_files"]
        check_item_types = options["check_item_types"]

        check_all = not any([
            check_ownership,
            check_files,
            check_item_types,
        ])

        if check_all or check_ownership:
            self.check_owners()

        if check_all or check_files:
            self.check_file_existence()

        if check_all or check_item_types:
            self.check_item_types()



from django import conf
from django.utils.module_loading import import_string

DEFAULTS = {
    "STORAGE": "django.core.files.storage.FileSystemStorage",
    "STORAGE_OPTIONS": {},
    "FILES_UPLOAD_TO": "files/%Y/%m/%d",
    "IMAGES_UPLOAD_TO": "images/%Y/%m/%d",
    "COLLECTION_FILES_UPLOAD_TO": "collections/files/%Y/%m/%d",
    "COLLECTION_IMAGES_UPLOAD_TO": "collections/images/%Y/%m/%d",
    "COLLECTION_ITEM_PREVIEW_WIDTH": 180,
    "COLLECTION_ITEM_PREVIEW_HEIGHT": 135,

    "RQ_ENABLED": False,
    "RQ_QUEUE_NAME": "default",
    "VARIATION_DEFAULTS": None,

    "CLOUDINARY_TYPE": "private",
    "CLOUDINARY_TEMP_DIR": "cloudinary",
    "CLOUDINARY_UPLOADER_OPTIONS": {
        "use_filename": True,
        "unique_filename": True,
        "overwrite": True,
        "invalidate": True
    },
}

# List of settings that may be in string import notation.
IMPORT_STRINGS = ("STORAGE",)

# Иконки для файлов в галерее
FILE_ICON_DEFAULT = "unknown"
FILE_ICON_OVERRIDES = {
    "3gp": "video",
    "aac": "audio",
    "docx": "doc",
    "flac": "audio",
    "flv": "video",
    "gz": "archive",
    "jpeg": "jpg",
    "m4a": "audio",
    "m4v": "video",
    "mkv": "video",
    "mov": "video",
    "mpeg": "video",
    "ogg": "audio",
    "ogv": "video",
    "xlsx": "xls",
    "xz": "archive",
    "wav": "audio",
    "webm": "video",
    "wma": "audio",
    "wmv": "video",
}


def perform_import(val, setting_name):
    """
    If the given setting is a string import notation,
    then perform the necessary import or imports.
    """
    if val is None:
        return None
    elif isinstance(val, str):
        return import_from_string(val, setting_name)
    elif isinstance(val, (list, tuple)):
        return [import_from_string(item, setting_name) for item in val]
    return val


def import_from_string(val, setting_name):
    """
    Attempt to import a class from a string representation.
    """
    try:
        return import_string(val)
    except ImportError as e:
        msg = "Could not import '%s' for API setting '%s'. %s: %s." % (
            val,
            setting_name,
            e.__class__.__name__,
            e,
        )
        raise ImportError(msg)


class Settings:
    """
    A settings object, that allows API settings to be accessed as properties.
    Any setting with string import paths will be automatically resolved
    and return the class, rather than the string literal.
    """

    def __init__(self, user_settings=None, defaults=None, import_strings=None):
        self.user_settings = user_settings
        self.defaults = defaults or DEFAULTS
        self.import_strings = import_strings or IMPORT_STRINGS

    def __getattr__(self, name):
        value = self._get_value(name)
        method_name = "prepare_{}".format(name.lower())
        prepare_method = getattr(self, method_name, None)
        if prepare_method is not None:
            value = prepare_method(value)
        self.__dict__[name] = value
        return value

    def _get_value(self, name):
        if name not in self.defaults:
            raise AttributeError("Invalid setting: '%s'" % name)

        try:
            # Check if present in user settings
            value = self.user_settings[name]
        except KeyError:
            # Fall back to defaults
            value = self.defaults[name]

        # Coerce import strings into classes
        if name in self.import_strings:
            return perform_import(value, name)
        return value


settings = Settings(
    getattr(conf.settings, "PAPER_UPLOADS", {}), DEFAULTS, IMPORT_STRINGS
)


IMAGE_ITEM_VARIATIONS = dict(
    admin_preview=dict(
        size=(
            settings.COLLECTION_ITEM_PREVIEW_WIDTH,
            settings.COLLECTION_ITEM_PREVIEW_HEIGHT
        ),
        format="jpeg",
        versions={"webp", "2x"},
        jpeg=dict(
            quality=75
        ),
        webp=dict(
            quality=65
        ),
    ),
)



from functools import partial

from django.template import Library
from jinja2_simple_tags import StandaloneTag

from ..helpers import paper_cloudinary_url

try:
    import jinja2
except ImportError:
    jinja2 = None

register = Library()


@register.simple_tag(takes_context=True, name="paper_cloudinary_url")
def do_paper_cloudinary_url(context, source, options_dict=None, **options):
    return paper_cloudinary_url(context, source, options_dict, **options)


if jinja2 is not None:
    class CloudinaryExtension(StandaloneTag):
        tags = {"paper_cloudinary_url"}

        def render(self, *args, **kwargs):
            return paper_cloudinary_url(self.context, *args, **kwargs)

    # django-jinja support
    try:
        from django_jinja import library
    except ImportError:
        pass
    else:
        dummy_ctx = {}
        cloudinary_url = partial(paper_cloudinary_url, dummy_ctx)
        library.global_function(name="paper_cloudinary_url")(cloudinary_url)
        library.extension(CloudinaryExtension)



from django.db import migrations, transaction
from django.db.models.signals import post_delete
from django.dispatch import receiver
from django.utils.timezone import now

from paper_uploads.models import CollectionItemBase


@receiver(post_delete, sender=CollectionItemBase)
def on_delete_collection_item(sender, instance, **kwargs):
    """
    Обновление поля `modified_at` коллекции при удалении элемента,
    чтобы метод `get_last_modified()` возвращал корректные данные.
    """
    if instance.collection_id and instance.collection_content_type_id:
        collection_cls = instance.get_collection_class()
        collection_cls.objects.filter(pk=instance.collection_id).update(
            modified_at=now()
        )


class RenameFileField(migrations.RunPython):
    def __init__(self, app_label, model_name, old_name, new_name):
        self.app_label = app_label
        self.model_name = model_name
        self.old_name = old_name
        self.new_name = new_name
        super().__init__(self.rename_forward, self.rename_backward)

    def _rename(self, apps, schema_editor, old_name, new_name):
        from ..models.fields.base import FileResourceFieldBase

        db = schema_editor.connection.alias
        state_model = apps.get_model(self.app_label, self.model_name)
        for field in state_model._meta.fields:
            if field.name != self.new_name:
                continue

            if isinstance(field, FileResourceFieldBase):
                with transaction.atomic(using=db):
                    field.related_model._base_manager.db_manager(db).filter(
                        owner_app_label=self.app_label,
                        owner_model_name=self.model_name,
                        owner_fieldname=old_name,
                    ).update(owner_fieldname=new_name)

    def rename_forward(self, apps, schema_editor):
        self._rename(apps, schema_editor, self.old_name, self.new_name)

    def rename_backward(self, apps, schema_editor):
        self._rename(apps, schema_editor, self.new_name, self.old_name)


class RenameFileModel(migrations.RunPython):
    def __init__(self, app_label, old_name, new_name):
        self.app_label = app_label
        self.old_name = old_name
        self.new_name = new_name
        super().__init__(self.rename_forward, self.rename_backward)

    def _rename(self, apps, schema_editor, old_name, new_name, backward=False):
        from ..models.fields.base import FileResourceFieldBase

        old_name = old_name.lower()
        new_name = new_name.lower()
        db = schema_editor.connection.alias
        state_model = apps.get_model(self.app_label, old_name if backward else new_name)
        for field in state_model._meta.fields:
            if isinstance(field, FileResourceFieldBase):
                with transaction.atomic(using=db):
                    field.related_model._base_manager.db_manager(db).filter(
                        owner_app_label=self.app_label,
                        owner_model_name=old_name,
                        owner_fieldname=field.name,
                    ).update(owner_model_name=new_name)

    def rename_forward(self, apps, schema_editor):
        self._rename(apps, schema_editor, self.old_name, self.new_name)

    def rename_backward(self, apps, schema_editor):
        self._rename(apps, schema_editor, self.new_name, self.old_name, backward=True)


def inject_rename_filefield_operations(plan=None, **kwargs):
    if plan is None:
        return

    for migration, backward in plan:
        inserts = []
        for index, operation in enumerate(migration.operations):
            if isinstance(operation, migrations.RenameField):
                operation = RenameFileField(
                    migration.app_label,
                    operation.model_name,
                    operation.old_name_lower,
                    operation.new_name_lower,
                )
                inserts.append((index + 1, operation))
            elif isinstance(operation, migrations.RenameModel):
                operation = RenameFileModel(
                    migration.app_label,
                    operation.old_name,
                    operation.new_name,
                )
                inserts.append((index + 1, operation))

        for inserted, (index, operation) in enumerate(inserts):
            migration.operations.insert(inserted + index, operation)



class CloudinaryOptionsMixin:
    def __init__(self, *args, upload_to="", cloudinary=None, **kwargs):
        self.cloudinary = cloudinary
        super().__init__(*args, upload_to=upload_to, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        if "cloudinary" in kwargs:
            del kwargs["cloudinary"]
        return name, path, args, kwargs



# Generated by Django 3.2.9 on 2021-11-06 10:04

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('paper_uploads_cloudinary', '0003_delete_cloudinarycollection'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='cloudinaryfileitem',
            options={'default_permissions': ()},
        ),
        migrations.AlterModelOptions(
            name='cloudinaryimageitem',
            options={'default_permissions': ()},
        ),
        migrations.AlterModelOptions(
            name='cloudinarymediaitem',
            options={'default_permissions': ()},
        ),
    ]



import logging
import sys
from datetime import timedelta

from anytree import LevelOrderIter
from django.core.management import color_style
from django.db import DEFAULT_DB_ALIAS
from django.utils.timezone import now

from .. import helpers
from . import utils


def remove_empty_collections(
    threshold: int = 24 * 3600,
    database: str = DEFAULT_DB_ALIAS,
    verbosity: int = logging.ERROR
):
    """
    Удаление коллекций, которые не содержат элементов.

    Удалять недавно созданные коллекции нельзя, т.к. они могут быть в процессе
    заполнения другим пользователем.

    :param threshold: минимальное время с момента создания коллекции в секундах
    :param database: алиас базы данных для поиска коллекций
    :param verbosity: уровень логирования
    :returns: количество удалённых коллекций
    """
    deleted = 0
    style = color_style()

    for root in helpers.get_collection_trees():
        for node in reversed(tuple(LevelOrderIter(root))):
            model = node.model

            created_before = now() - timedelta(seconds=threshold)
            queryset = model.objects.using(database).filter(
                items=None,
                created_at__lte=created_before
            )

            count = queryset.count()
            if not count:
                continue

            if verbosity <= logging.DEBUG:
                print(
                    "Removing {count} {verb} of {target} ...".format(
                        count=style.WARNING(count),
                        verb="instance" if count == 1 else "instances",
                        target=style.NOTICE(
                            "{}.{}".format(
                                model._meta.app_label,
                                model.__name__,
                            )
                        )
                    ),
                    end=" "
                )

            queryset.delete()

            if verbosity <= logging.DEBUG:
                print(style.WARNING("ok"))

            deleted += count

    return deleted


def create_missing_variations(
    async_: bool = False,
    database: str = DEFAULT_DB_ALIAS,
    verbosity: int = logging.ERROR
):
    """
    Создание отсутствующих файлов вариаций.

    :param async_: использовать django-rq для создания файлов вариаций
    :param database: алиас базы данных для поиска файлов
    :param verbosity: уровень логирования
    :returns: количество созданных файлов
    """
    variations_created = 0
    style = color_style()

    for node in helpers.get_resource_model_trees():
        model = node.model

        if not utils.is_variations_allowed(model):
            continue

        for instance in model.objects.using(database).iterator():

            missing_variations = []
            for vname, vfile in instance.variation_files():
                if vfile is not None and not vfile.exists():
                    missing_variations.append(vname)

            if missing_variations and instance.file_exists():
                count = len(missing_variations)

                if verbosity <= logging.DEBUG:
                    print(
                        "Creating {count} {verb} for {target} ...".format(
                            count=style.WARNING(count),
                            verb="variation" if count == 1 else "variations",
                            target=style.NOTICE(
                                "{}.{} #{}".format(
                                    model._meta.app_label,
                                    model.__name__,
                                    instance.pk
                                )
                            )
                        ),
                        end=" "
                    )
                    sys.stdout.flush()

                if async_:
                    instance.recut_async(names=missing_variations)
                else:
                    instance.recut(names=missing_variations)

                if verbosity <= logging.DEBUG:
                    print(style.WARNING("ok"))
                    sys.stdout.flush()

                variations_created += count

    return variations_created



from typing import List, Type

from django.db import models
from django.db.models.fields import Field

from ..helpers import build_variations
from ..models.base import VersatileImageResourceMixin
from ..models.collection import CollectionBase
from ..models.fields.collection import CollectionItem


def is_variations_allowed(model: Type[models.Model]) -> bool:
    """
    Возвращает True, если модель представляет из себя класс изображения с вариациями.
    """
    return issubclass(model, VersatileImageResourceMixin)


def is_collection(model: Type[models.Model]) -> bool:
    """
    Возвращает True, если model - коллекция.
    """
    return issubclass(model, CollectionBase)


def get_field_variations(field: Field) -> List[str]:
    """
    Получение списка имен вариаций для поля.
    """
    return list(
        field.variations.keys()
    )


def get_collection_variations(collection_cls: Type[CollectionBase], item_type_field: CollectionItem) -> List[str]:
    """
    Получение списка имен вариаций для коллекции.
    """
    image_model = item_type_field.model
    variation_config = image_model.get_variation_config(collection_cls, item_type_field)
    variations = build_variations(variation_config)
    return list(variations.keys())



import hashlib
import re
from typing import Any, Dict, Iterable, Set, Tuple

from django.utils import formats
from django.utils.html import avoid_wrapping
from django.utils.translation import gettext, ngettext

from .typing import FileLike

filesize_regex = re.compile(r"^([.\d]+)\s*([KMGT])?B?$")
filesize_units = {"K": 10 ** 3, "M": 10 ** 6, "G": 10 ** 9, "T": 10 ** 12}


def checksum(file: FileLike) -> str:
    """
    DropBox checksum realization.
    https://www.dropbox.com/developers/reference/content-hash
    """
    if file.closed:
        file.open("rb")
    elif file.seekable():
        file.seek(0)

    blocks = []
    while True:
        data = file.read(4 * 1024 * 1024)
        if not data:
            break
        blocks.append(hashlib.sha256(data).digest())
    return hashlib.sha256(b"".join(blocks)).hexdigest()


def remove_dulpicates(seq: Iterable) -> Tuple:
    """
    https://stackoverflow.com/questions/480214/how-do-you-remove-duplicates-from-a-list-whilst-preserving-order
    """
    seen = set()  # type: Set[Any]
    seen_add = seen.add
    return tuple(x for x in seq if not (x in seen or seen_add(x)))


def lowercased_dict_keys(options: Dict[str, Any]) -> Dict[str, Any]:
    """
    Возвращает копию словаря с ключами, приведенными к нижнему регистру.
    """
    return {key.lower(): value for key, value in options.items()}


def parse_filesize(value: str) -> int:
    """
    Парсинг человеко-понятного значения размера файла.
    Допустимые форматы значения: 4k, 4kb, 4KB, 4 K, 4 Kb
    """
    match = filesize_regex.match(value.upper().strip())
    number, unit = match.groups()
    return int(float(number) * filesize_units.get(unit, 1))


def filesizeformat(bytes_: int) -> str:
    """
    Форматирование размера файла.
    В отличие от одноименной встроенной функции, выводит настоящие килобайты/мегабайты,
    вместо кибибайтов/мибибайтов.
    """
    try:
        bytes_ = float(bytes_)
    except (TypeError, ValueError, UnicodeDecodeError):
        value = ngettext("%(size)d byte", "%(size)d bytes", 0) % {"size": 0}
        return avoid_wrapping(value)

    def filesize_number_format(value):
        return formats.number_format(round(value, 1), 1)

    KB = 10 ** 3
    MB = 10 ** 6
    GB = 10 ** 9
    TB = 10 ** 12
    PB = 10 ** 15

    negative = bytes_ < 0
    if negative:
        bytes_ = -bytes_  # Allow formatting of negative numbers.

    if bytes_ < KB:
        value = ngettext("%(size)d byte", "%(size)d bytes", bytes_) % {"size": bytes_}
    elif bytes_ < MB:
        value = gettext("%s KB") % filesize_number_format(bytes_ / KB)
    elif bytes_ < GB:
        value = gettext("%s MB") % filesize_number_format(bytes_ / MB)
    elif bytes_ < TB:
        value = gettext("%s GB") % filesize_number_format(bytes_ / GB)
    elif bytes_ < PB:
        value = gettext("%s TB") % filesize_number_format(bytes_ / TB)
    else:
        value = gettext("%s PB") % filesize_number_format(bytes_ / PB)

    if negative:
        value = "-%s" % value
    return avoid_wrapping(value)



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class Config(AppConfig):
    label = "paper_uploads_cloudinary"
    name = "paper_uploads.cloudinary"
    verbose_name = _("Uploaded Files")



# Generated by Django 4.0.1 on 2022-02-17 14:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('paper_uploads_cloudinary', '0007_alter_cloudinaryfile_resource_name_and_more'),
    ]

    run_before = [
        ('paper_uploads', '0004_remove_fileitem_created_at_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='cloudinaryfileitem',
            name='created_at',
        ),
        migrations.RemoveField(
            model_name='cloudinaryfileitem',
            name='modified_at',
        ),
        migrations.RemoveField(
            model_name='cloudinaryimageitem',
            name='created_at',
        ),
        migrations.RemoveField(
            model_name='cloudinaryimageitem',
            name='modified_at',
        ),
        migrations.RemoveField(
            model_name='cloudinarymediaitem',
            name='created_at',
        ),
        migrations.RemoveField(
            model_name='cloudinarymediaitem',
            name='modified_at',
        ),
    ]



import datetime
from collections import defaultdict
from datetime import timedelta
from typing import Type

from django.apps import apps
from django.core.management import BaseCommand
from django.db import DEFAULT_DB_ALIAS
from django.utils.timezone import now

from ... import helpers
from ...models.base import FileResource
from ...models.collection import CollectionBase
from ...models.mixins import BacklinkModelMixin


class Command(BaseCommand):
    help = """
    Находит некорректные экземпляры файловых моделей и предлагает их удалить.
    
    Некорректными считаются экземпляры, у которых утерян загруженный файл,
    а также экземпляры, на которые нет ссылки.
    
    Создание экземпляра файловой модели и загрузка в неё файла - это  две отдельные 
    операции. Между ними может пройти какое-то время, особенно при использовании 
    django-rq. Для того, чтобы сохранить экземпляры, в которые ещё не загрузились файлы,
    эта команда пропускает те экземпляры, которые созданы недавно.  Временной интервал 
    задаётся через параметр `--threshold` и по умолчанию составляет 24 часа.
    """
    verbosity = None
    database = DEFAULT_DB_ALIAS
    interactive = True

    def add_arguments(self, parser):
        parser.add_argument(
            "--database",
            action="store",
            dest="database",
            default=DEFAULT_DB_ALIAS,
            help="Nominates the database to use. Defaults to the 'default' database.",
        )
        parser.add_argument(
            "-o", "--check-ownership",
            action="store_true",
            default=False,
            help="Check `owner_XXX` fields.",
        )
        parser.add_argument(
            "-f", "--check-files",
            action="store_true",
            default=False,
            help="Check file existence.",
        )
        parser.add_argument(
            "--noinput",
            "--no-input",
            action="store_false",
            dest="interactive",
            help="Do NOT prompt the user for input of any kind.",
        )
        parser.add_argument(
            "--threshold",
            type=int,
            default=24 * 3600,
            help="Minimum instance age in seconds to look for",
        )

    def _get_start_time(self) -> datetime.datetime:
        return now() - timedelta(seconds=self.options["threshold"])

    def _clean_objects_with_invalid_ownership(self, model: Type[BacklinkModelMixin]):
        grouped_invalid_objects = defaultdict(list)

        queryset = model.objects.using(self.database).filter(created_at__lte=self._get_start_time())
        for instance in queryset.iterator():
            owner_model = instance.get_owner_model()
            if owner_model is None:
                model = type(instance)
                grouped_invalid_objects[model].append(instance)
            else:
                owner_field = instance.get_owner_field()
                if owner_field is None:
                    model = type(instance)
                    grouped_invalid_objects[model].append(instance)
                else:
                    try:
                        owner_model._base_manager.using(self.database).get(
                            (instance.owner_fieldname, instance.pk)
                        )
                    except owner_model.DoesNotExist:
                        model = type(instance)
                        grouped_invalid_objects[model].append(instance)

        if not grouped_invalid_objects:
            return

        if self.interactive:
            for actual_model, instances in grouped_invalid_objects.items():
                while True:
                    print("-" * 64)
                    answer = input(
                        "Found \033[92m%d '%s.%s'\033[0m objects with invalid ownership.\n"
                        "IDs: %s\n"
                        "What would you like to do with them?\n"
                        "(p)rint / (k)eep / (d)elete [default=keep]? "
                        % (
                            len(instances),
                            actual_model._meta.app_label,
                            actual_model.__name__,
                            ", ".join(map(str, [obj.pk for obj in instances])),
                        )
                    )
                    answer = answer.lower() or "k"
                    if answer in {"p", "print"}:
                        self.stdout.write("\n")

                        sorted_objects = sorted(instances, key=lambda obj: obj.pk)
                        for index, obj in enumerate(sorted_objects, start=1):
                            prefix = "{})".format(index)
                            self.stdout.write(
                                "{:<3} \033[92m{}.{}\033[0m (ID: {})\n"
                                "    File: {}".format(
                                    prefix,
                                    actual_model._meta.app_label,
                                    actual_model.__name__,
                                    obj.pk,
                                    obj.name,
                                )
                            )
                        self.stdout.write("\n")
                    elif answer in {"k", "keep"}:
                        break
                    elif answer in {"d", "delete"}:
                        model.objects.using(self.database).filter(
                            pk__in=[obj.pk for obj in instances]
                        ).delete()
                        break
        else:
            for actual_model, instances in grouped_invalid_objects.items():
                model.objects.using(self.database).filter(
                    pk__in=[obj.pk for obj in instances]
                ).delete()

    def clean_objects_with_invalid_ownership(self):
        if self.verbosity >= 2:
            self.stdout.write(self.style.SUCCESS("Checking resource ownership..."))

        for node in helpers.get_resource_model_trees(include_proxy=False):
            model = node.model

            if not issubclass(model, BacklinkModelMixin):
                continue

            self._clean_objects_with_invalid_ownership(model)

        for model in apps.get_models():
            if not issubclass(model, CollectionBase):
                continue

            if model._meta.proxy:
                continue

            self._clean_objects_with_invalid_ownership(model)

    def _clean_objects_with_missing_files(self, model: Type[FileResource]):
        """
        Поиск экземпляров с утерянными файлами
        """
        grouped_invalid_objects = defaultdict(list)

        queryset = model.objects.using(self.database).filter(created_at__lte=self._get_start_time())
        for instance in queryset.iterator():
            if not instance.file_exists():
                actual_model = type(instance)  # support polymophic models
                grouped_invalid_objects[model].append(actual_model)

        if not grouped_invalid_objects:
            return

        if self.interactive:
            for actual_model, instances in grouped_invalid_objects.items():
                while True:
                    print("-" * 64)
                    answer = input(
                        "Found \033[92m%d '%s.%s'\033[0m objects that refers to a file that does not exist.\n"
                        "IDs: %s\n"
                        "What would you like to do with them?\n"
                        "(p)rint / (k)eep / (d)elete [default=keep]? "
                        % (
                            len(instances),
                            actual_model._meta.app_label,
                            actual_model.__name__,
                            ", ".join(map(str, [obj.pk for obj in instances])),
                        )
                    )
                    answer = answer.lower() or "k"
                    if answer in {"p", "print"}:
                        self.stdout.write("\n")

                        sorted_objects = sorted(instances, key=lambda obj: obj.pk)
                        for index, obj in enumerate(sorted_objects, start=1):
                            prefix = "{})".format(index)
                            self.stdout.write(
                                "{:<3} \033[92m{}.{}\033[0m (ID: {})\n"
                                "    File: {}".format(
                                    prefix,
                                    actual_model._meta.app_label,
                                    actual_model.__name__,
                                    obj.pk,
                                    obj.name,
                                )
                            )
                        self.stdout.write("\n")
                    elif answer in {"k", "keep"}:
                        break
                    elif answer in {"d", "delete"}:
                        model.objects.using(self.database).filter(
                            pk__in=[obj.pk for obj in instances]
                        ).delete()
                        break
        else:
            for actual_model, instances in grouped_invalid_objects.items():
                model.objects.using(self.database).filter(
                    pk__in=[obj.pk for obj in instances]
                ).delete()

    def clean_objects_with_missing_files(self):
        if self.verbosity >= 2:
            self.stdout.write(self.style.SUCCESS("Checking file existence..."))

        for node in helpers.get_resource_model_trees(include_proxy=False):
            model = node.model

            if not issubclass(model, FileResource):
                continue

            self._clean_objects_with_missing_files(model)

    def handle(self, *args, **options):
        self.options = options
        self.verbosity = options["verbosity"]
        self.database = options["database"]
        self.interactive = options["interactive"]

        check_ownership = options["check_ownership"]
        check_files = options["check_files"]

        check_all = not any([
            check_ownership,
            check_files,
        ])

        if check_all or check_ownership:
            self.clean_objects_with_invalid_ownership()

        if check_all or check_files:
            self.clean_objects_with_missing_files()



from .... import forms
from ....models.fields.base import FileResourceFieldBase
from .base import CloudinaryOptionsMixin


class CloudinaryImageField(CloudinaryOptionsMixin, FileResourceFieldBase):
    def __init__(self, *args, **kwargs):
        kwargs.setdefault("to", "paper_uploads_cloudinary.CloudinaryImage")
        super().__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        return super().formfield(**{"form_class": forms.ImageField, **kwargs})



import datetime
import os
import posixpath
import tempfile
from io import UnsupportedOperation
from typing import Optional

import cloudinary.exceptions
import requests
from cloudinary import CloudinaryResource, uploader
from cloudinary.utils import upload_params
from django.core.exceptions import ValidationError
from django.core.files import File
from django.core.files.utils import FileProxyMixin
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _
from filelock import FileLock

from ... import exceptions, helpers, utils
from ...conf import settings
from ...logging import logger
from ...models.base import FileFieldResource
from ...models.mixins import BacklinkModelMixin
from .mixins import ReadonlyCloudinaryFileProxyMixin


class CloudinaryFieldFile(FileProxyMixin):
    """
    Реализация интерфейса django.core.files.File для Cloudinary
    """
    DEFAULT_CHUNK_SIZE = 64 * 2 ** 10

    def __init__(self, resource: CloudinaryResource, checksum: str = None):
        self.resource = resource
        self.name = self.public_id
        self.checksum = checksum
        self.file = None

    def __str__(self):
        return self.name or ""

    def __repr__(self):
        return "<%s: %s>" % (self.__class__.__name__, self or "None")

    def __bool__(self):
        return bool(self.name)

    def __len__(self):
        return self.size

    def __eq__(self, other):
        return self.url == other.url

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, tb):
        self.close()

    @cached_property
    def public_id(self):
        public_id = self.resource.public_id
        if (self.resource.resource_type == "raw") and self.resource.format:
            public_id += "." + self.resource.format
        return public_id

    @cached_property
    def metadata(self):
        if self.resource.metadata is not None:
            return self.resource.metadata

        return uploader.explicit(
            self.public_id,
            type=self.resource.type,
            resource_type=self.resource.resource_type
        )

    @property
    def url(self) -> str:
        return self.resource.url

    @property
    def size(self) -> int:
        return self.metadata["bytes"]

    @property
    def format(self) -> Optional[str]:
        return self.metadata.get("format")

    def _get_tempfile_path(self):
        return os.path.join(
            tempfile.gettempdir(),
            settings.CLOUDINARY_TEMP_DIR,
            self.name
        )

    def _download_file(self, mode="rb", chunk_size=None):
        """
        Скачивание файла из Cloudinary во временную директорию.
        Загруженный файл остается там даже после закрытия, чтобы не
        скачивать его заново в следующий раз.
        """
        chunk_size = chunk_size or self.DEFAULT_CHUNK_SIZE
        tempfile_path = self._get_tempfile_path()
        if os.path.exists(tempfile_path):
            with open(tempfile_path, "rb") as fp:
                file_checksum = utils.checksum(fp)
            if file_checksum == self.checksum:
                return File(open(tempfile_path, mode))

        root, basename = os.path.split(tempfile_path)
        os.makedirs(root, mode=0o755, exist_ok=True)

        lock = FileLock(tempfile_path + ".lock")
        with lock.acquire(timeout=3600):
            response = requests.get(self.url, stream=True)
            response.raise_for_status()
            with open(tempfile_path, "wb+") as fp:
                for chunk in response.iter_content(chunk_size=chunk_size):
                    fp.write(chunk)
            return File(open(tempfile_path, mode))

    def open(self, mode="rb"):
        if self.file is None:
            self.file = self._download_file(mode)
        elif self.file.closed:
            self.file.open(mode)
        else:
            self.seek(0)
        return self

    def read(self, size=None):
        if not self.closed:
            return self.file.read(size)
        raise ValueError("Cannot read from file.")

    def write(self, chunk):
        if not self.closed:
            return self.file.write(chunk)
        raise ValueError("Cannot write to file.")

    def close(self):
        self.file.close()

    def multiple_chunks(self, chunk_size=None):
        return self.size > (chunk_size or self.DEFAULT_CHUNK_SIZE)

    def chunks(self, chunk_size=None):
        chunk_size = chunk_size or self.DEFAULT_CHUNK_SIZE
        try:
            self.seek(0)
        except (AttributeError, UnsupportedOperation):
            pass

        while True:
            data = self.read(chunk_size)
            if not data:
                break
            yield data


class CloudinaryFileFieldResourceMixin(ReadonlyCloudinaryFileProxyMixin):
    class Meta:
        abstract = True

    def file_exists(self) -> bool:
        file = self.get_file()
        if not file:
            return False

        file_field = self.get_file_field()

        try:
            uploader.explicit(
                self.name,
                type=file_field.type,
                resource_type=file_field.resource_type,
            )
        except cloudinary.exceptions.Error:
            return False
        return True

    def get_cloudinary_options(self):
        options = (settings.CLOUDINARY_UPLOADER_OPTIONS or {}).copy()

        file_field = self.get_file_field()
        options.update(file_field.options)

        # owner field`s options should override `CloudinaryField` options
        if isinstance(self, BacklinkModelMixin):
            owner_field = self.get_owner_field()
            if owner_field is not None and hasattr(owner_field, "cloudinary"):
                options.update(owner_field.cloudinary or {})

        # filter keys
        options = {
            key: value
            for key, value in options.items()
            if key in upload_params + uploader.upload_options
        }

        # format folder
        folder = options.pop("folder", None)
        if folder is None:
            folder = self.get_file_folder()
        if isinstance(folder, str):
            options["folder"] = datetime.datetime.now().strftime(folder)

        return options

    def _attach(self, file: File, **options):
        cloudinary_options = self.get_cloudinary_options()
        cloudinary_options.update(options.get("cloudinary", {}))

        if file.size >= 100 * 1024 * 1024:
            upload = uploader.upload_large
        else:
            upload = uploader.upload

        file_field = self.get_file_field()
        cloudinary_options.setdefault("type", file_field.type)
        cloudinary_options.setdefault("resource_type", file_field.resource_type)

        try:
            result = upload(
                file,
                **cloudinary_options,
            )
        except cloudinary.exceptions.Error as e:
            if e.args and "Unsupported" in e.args[0]:
                raise exceptions.UnsupportedResource(
                    _("File `%s` may be corrupt, or in an unsupported format.") % file.name
                )
            else:
                raise ValidationError(*e.args)

        # fix difference between `public_id` in response
        # and `public_id` in CloudinaryField
        file_format = result.get("format")
        if file_format:
            public_id = result["public_id"]
        else:
            public_id, file_format = posixpath.splitext(result["public_id"])
            file_format = file_format.lstrip(".")

        resource = CloudinaryResource(
            public_id,
            version=str(result["version"]),
            format=file_format,
            type=result["type"],
            resource_type=result["resource_type"],
            metadata=result,
        )

        self.set_file(resource)

        self.resource_name = helpers.get_filename(file.name)
        self.extension = file_format or ""
        return result

    def _rename(self, new_name: str, **options):
        # TODO: Cloudinary can't copy files. We dont't want to do it manually
        cloudinary_options = self.get_cloudinary_options()
        cloudinary_options.update(options.get("cloudinary", {}))

        file_field = self.get_file_field()
        cloudinary_options.setdefault("type", file_field.type)
        cloudinary_options.setdefault("resource_type", file_field.resource_type)

        old_name = self.name
        folder, basename = posixpath.split(old_name)
        if cloudinary_options["resource_type"] != "raw":
            # video and image have no extension
            base, ext = posixpath.splitext(new_name)
            new_name = base
        new_name = posixpath.join(folder, new_name)

        try:
            result = uploader.rename(
                old_name,
                new_name,
                **cloudinary_options
            )
        except cloudinary.exceptions.Error as e:
            raise ValidationError(*e.args)

        # fix difference between `public_id` in response
        # and `public_id` in CloudinaryField
        file_format = result.get("format")
        if file_format:
            public_id = result["public_id"]
        else:
            public_id, file_format = posixpath.splitext(result["public_id"])
            file_format = file_format.lstrip(".")

        resource = CloudinaryResource(
            public_id,
            version=str(result["version"]),
            format=file_format,
            type=result["type"],
            resource_type=result["resource_type"],
            metadata=result,
        )
        self.set_file(resource)

        self.resource_name = helpers.get_filename(new_name)
        self.extension = file_format or ""
        return result

    def _delete_file(self, **options):
        cloudinary_options = self.get_cloudinary_options()
        cloudinary_options.update(options.get("cloudinary", {}))

        file = self.get_file()
        if not file:
            return

        file_field = self.get_file_field()
        cloudinary_options.setdefault("type", file_field.type)
        cloudinary_options.setdefault("resource_type", file_field.resource_type)

        try:
            result = uploader.destroy(
                self.name,
                **cloudinary_options
            )
        except cloudinary.exceptions.Error:
            logger.exception(
                "Couldn't delete Cloudinary file: {}".format(self.name)
            )
            return

        status = result.get("result")
        if status == "ok":
            self.set_file(None)
        else:
            logger.warning(
                "Unable to delete Cloudinary file `{}`: {}".format(
                    self.name, status
                )
            )
        return result

    def build_url(self, **options):
        # proxy Cloudinary method
        self._require_file()
        file = self.get_file()
        return file.resource.build_url(**options)



from ...models.mixins import FileProxyMixin


class ReadonlyCloudinaryFileProxyMixin(FileProxyMixin):
    """
    Проксирование некоторых свойств файла (только для чтения) на уровень модели

    TODO: удалить _require_file?
    """

    @property
    def url(self):
        self._require_file()  # noqa: F821
        return self.get_file().url  # noqa: F821



# Generated by Django 3.1.8 on 2021-04-07 06:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('paper_uploads_cloudinary', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='cloudinarycollection',
            options={'default_permissions': ()},
        ),
        migrations.AlterModelOptions(
            name='cloudinaryfileitem',
            options={'default_permissions': (), 'verbose_name': 'File item', 'verbose_name_plural': 'File items'},
        ),
        migrations.AlterModelOptions(
            name='cloudinaryimagecollection',
            options={'default_permissions': ()},
        ),
        migrations.AlterModelOptions(
            name='cloudinaryimageitem',
            options={'default_permissions': (), 'verbose_name': 'Image item', 'verbose_name_plural': 'Image items'},
        ),
        migrations.AlterModelOptions(
            name='cloudinarymediaitem',
            options={'default_permissions': (), 'verbose_name': 'Media item', 'verbose_name_plural': 'Media items'},
        ),
    ]



import sys
from typing import Iterable, Type

from django.apps import apps
from django.core.management import BaseCommand
from django.db import DEFAULT_DB_ALIAS

from ...models.base import Resource
from ...models.collection import CollectionBase
from .. import utils


class Command(BaseCommand):
    help = """
    Удаление вариаций для всех экземпляров указанной модели.
    
    Пример для обычной модели:
        python3 manage.py remove_variations blog.post --field=hero
    
    Пример для коллекции:
        python3 manage.py remove_variations blog.gallery --item-type=image
    """
    options = None
    verbosity = None
    database = DEFAULT_DB_ALIAS
    interactive = False

    def add_arguments(self, parser):
        parser.add_argument(
            "model",
            metavar="app_label.ModelName",
            help="Specifies the model to remove variations from",
        )
        parser.add_argument(
            "--field",
            nargs="?",
            help="Restricts variations to the specified field. "
                 "You should not specify any field for Collection models.",
        )
        parser.add_argument(
            "--item-type",
            nargs="?",
            help="Only look for variations in the specified CollectionItem. "
                 "Use this argument for Collection models only.",
        )
        parser.add_argument(
            "--variations",
            dest="variations",
            nargs="+",
            help="Specifies the variation names to delete",
        )
        parser.add_argument(
            "-i", "--interactive",
            action="store_true",
        )
        parser.add_argument(
            "--database",
            action="store",
            dest="database",
            default=DEFAULT_DB_ALIAS,
            help="Nominates the database to use. Defaults to the 'default' database.",
        )

    def handle(self, *args, **options):
        self.options = options
        self.verbosity = options["verbosity"]
        self.database = options["database"]
        self.interactive = options["interactive"]

        model = apps.get_model(self.options["model"])
        if utils.is_collection(model):
            self.process_collection(model)
        else:
            self.process_resource(model)

    def process_collection(self, model: Type[CollectionBase]):
        item_type = self.options["item_type"]
        if not item_type:
            raise RuntimeError("The argument 'item-type' is required")

        if item_type not in model.item_types:
            raise RuntimeError("Unsupported collection item type: %s" % item_type)

        item_type_field = model.item_types[item_type]
        if not utils.is_variations_allowed(item_type_field.model):
            raise RuntimeError("Specified collection item type has no variations: %s" % item_type)

        variations = self.options["variations"]
        if not variations:
            if self.interactive:
                variations = self.variations_dialog(model, item_type_field)

        if not variations:
            variations = set(utils.get_collection_variations(model, item_type_field))

        self._process_collection(model, item_type, variations)

    def _process_collection(self, model: Type[CollectionBase], item_type: str, variations: Iterable[str]):
        queryset = model.objects.using(self.database)

        total = queryset.count()
        for index, collection in enumerate(queryset.iterator(), start=1):
            if self.verbosity >= 1:
                self.stdout.write(
                    "Remove variations for \033[92m'{}.{}'\033[0m (ID: {}) ({}/{}) ... ".format(
                        model._meta.app_label,
                        model.__name__,
                        collection.pk,
                        index,
                        total
                    )
                )

            for item in collection.get_items(item_type).iterator():
                for variation_name in item.get_variations():
                    if variation_name in variations:
                        variation_file = item.get_variation_file(variation_name)
                        variation_file.delete()

    def process_resource(self, model: Type[Resource]):
        fieldname = self.options["field"]
        if not fieldname:
            raise RuntimeError("The argument 'field' is required")

        field = model._meta.get_field(fieldname)
        if field.is_relation and not utils.is_variations_allowed(field.related_model):
            raise RuntimeError("Specified field has no variations: %s" % fieldname)

        variations = self.options["variations"]
        if not variations:
            if self.interactive:
                variations = self.variations_dialog(model, field)

        if not variations:
            variations = set(utils.get_field_variations(field))

        self._process_resource(model, fieldname, variations)

    def _process_resource(self, model: Type[Resource], fieldname: str, variations: Iterable[str]):
        queryset = model.objects.using(self.database).exclude((fieldname, None))

        total = queryset.count()
        for index, instance in enumerate(queryset.iterator(), start=1):
            if self.verbosity >= 1:
                self.stdout.write(
                    "Remove variations for \033[92m'{}.{}'\033[0m (ID: {}) ({}/{}) ... ".format(
                        type(instance)._meta.app_label,
                        type(instance).__name__,
                        instance.pk,
                        index,
                        total
                    )
                )
            field = getattr(instance, fieldname)

            for variation_name in variations:
                if variation_name in field.get_variations():
                    variation_file = field.get_variation_file(variation_name)
                    variation_file.delete()

    def variations_dialog(self, model, field):
        if utils.is_collection(model):
            allowed_variations = utils.get_collection_variations(model, field)
        else:
            allowed_variations = utils.get_field_variations(field)

        while True:
            self.stdout.write(
                self.style.SUCCESS(
                    "Please, specify the variations you would like to process."
                )
            )
            self.stdout.write(" {}) All variations".format(self.style.SUCCESS("*")))
            for index, vname in enumerate(allowed_variations, start=1):
                self.stdout.write(
                    "{}) Variation `{}`".format(
                        self.style.SUCCESS("{:>2}".format(index)), vname
                    )
                )
            self.stdout.write(" {}) Abort".format(self.style.SUCCESS("q")))
            self.stdout.write("Enter your choice: ", ending="")

            answer_string = input().strip()
            answers = tuple(map(str.strip, answer_string.split(",")))
            if "*" in answers:
                return allowed_variations
            if "q" in answers:
                sys.exit()

            try:
                answers = tuple(map(int, answers))
            except ValueError:
                self.stderr.write(
                    "Invalid selection. Press Enter to try again... ", ending=""
                )
                input()
                continue

            if not all(1 <= answer <= len(allowed_variations) for answer in answers):
                self.stderr.write(
                    "Invalid selection. Press Enter to try again... ", ending=""
                )
                input()
                continue

            return set(allowed_variations[answer - 1] for answer in answers)



from .collection import (
    CloudinaryFileItemBase,
    CloudinaryImageItemBase,
    CloudinaryMediaItemBase,
    CloudinaryFileItem,
    CloudinaryImageItem,
    CloudinaryMediaItem,
    CloudinaryImageCollection,
)
from .fields import CloudinaryFileField, CloudinaryImageField, CloudinaryMediaField
from .file import CloudinaryFile
from .image import CloudinaryImage
from .media import CloudinaryMedia

__all__ = [
    "CloudinaryFile",
    "CloudinaryImage",
    "CloudinaryMedia",
    "CloudinaryFileField",
    "CloudinaryImageField",
    "CloudinaryMediaField",
    "CloudinaryFileItemBase",
    "CloudinaryMediaItemBase",
    "CloudinaryImageItemBase",
    "CloudinaryFileItem",
    "CloudinaryImageItem",
    "CloudinaryMediaItem",
    "CloudinaryImageCollection",
]



# Generated by Django 3.2.9 on 2021-11-16 08:40

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('paper_uploads_cloudinary', '0004_auto_20211106_1004'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='cloudinaryfileitem',
            name='owner_app_label',
        ),
        migrations.RemoveField(
            model_name='cloudinaryfileitem',
            name='owner_fieldname',
        ),
        migrations.RemoveField(
            model_name='cloudinaryfileitem',
            name='owner_model_name',
        ),
        migrations.RemoveField(
            model_name='cloudinaryimageitem',
            name='owner_app_label',
        ),
        migrations.RemoveField(
            model_name='cloudinaryimageitem',
            name='owner_fieldname',
        ),
        migrations.RemoveField(
            model_name='cloudinaryimageitem',
            name='owner_model_name',
        ),
        migrations.RemoveField(
            model_name='cloudinarymediaitem',
            name='owner_app_label',
        ),
        migrations.RemoveField(
            model_name='cloudinarymediaitem',
            name='owner_fieldname',
        ),
        migrations.RemoveField(
            model_name='cloudinarymediaitem',
            name='owner_model_name',
        ),
    ]
